
var Navigate = Navigate || {};

Navigate.loaderCount = 0;

Drupal.behaviors.navigate = function () {
  
  Navigate.bindToggleButtons();
  Navigate.bindCallbackButtons();
  Navigate.bindTextInputs();
  Navigate.bindAdminTools();
  Navigate.bindTextareas();
  Navigate.bindAddWidgetList();
  
  
  // Run one-time bindings on Navigate area
  if (!$('.navigate').hasClass('navigate-processed')) {
    $('.navigate').addClass('navigate-processed')
    Navigate.bindNavigateOneTime();
    $(document).bind('keydown', {combi:'ctrl+shift+n'}, Navigate.toggle);
    if ($('.navigate-key-button').hasClass('key-disabled')) {
      $(document).unbind('keydown', {combi:'ctrl+shift+n'});
    }
  }


  // Bind event to widget settings button
  $('.navigate-widget-settings-button').each(function () {
    if ($(this).hasClass('widget-settings-button-processed')) {
      return;
    }
    $(this).addClass('widget-settings-button-processed');
    $(this).click(function() {
      $(this).parent().find('.navigate-widget-settings-outer').slideToggle('fast');
    });
  });
  
  
  // Add select all on text inputs
  $('.navigate-select-all').each(function () {
    if ($(this).hasClass('select-all-processed')) {
      return;
    }
    $(this).addClass('select-all-processed');
    $(".navigate-select-all").focus(function () {
      $(this).select();
    });
  });


  // Add tooltips - v.2
  $('.navigate-tooltip').each(function () {
    if ($(this).hasClass('tooltip-processed')) {
      return;
    }
    $(this).addClass('tooltip-processed');
    $content = $('#' + $(this).attr('id') + '_tip_content').html();
    if ($content != '') {
      $(this).tooltip({
        bodyHandler: function() {
          return $('#' + $(this).attr('id') + '_tip_content').html();
        },
        track: true,
        fixPNG: true,
        fade: 150,
        delay:750
      });
    }
  });

  // Bind widget close button
  $('.navigate-widget-close').click(function() {
    if ($(this).hasClass('widget-close-processed')) {
      return;
    }
    $(this).addClass('widget-close-processed');
    var wid = $(this).find('.navigate-widget-close-id').val();
    
    Navigate.queueAdd();
    $.ajax({
        url: Navigate.ajaxUrl(),
        type: 'POST',
        data: 'action=widget_delete&wid=' + wid,
        error: function() {
        },
        success: function(msg) {
          $('#navigate-widget-outer-' + wid).slideUp('fast', function() {
            $('#navigate-widget-outer-' + wid).remove();
            Navigate.queueSubtract();
          });
        }
      });
  });
  
  
  // Add double-click to widget title to change it
  $(".navigate-widget-title-customizable").each(function () {
    if ($(this).hasClass('navigate-widget-title-processed')) {
      return;
    }
    $(this).addClass('navigate-widget-title-processed');
    $(this).dblclick(function() {
      var parent = $(this).parent();
      var wid = $(parent).find('.navigate-title-wid').val();
      $(parent).html('<input type="text" class="navigate-title-input" value="' + $(this).text() + '" />');
      $(parent).children().select();
      $(parent).children().bind('keyup', function(e) {
        // If pressing enter
        if (e.which == 13 || e.keyCode == 13) {
          if ($(parent).children().val() == '') {
            alert('Please enter some value for the title');
          } else {
            Navigate.variableSet('widget_title', $(parent).children().val(), wid);
            $(parent).html('<div class="navigate-widget-title">' + $(parent).children().val() + '</div>');
            var parent_id = $(parent).attr('id');
            Drupal.attachBehaviors('#' + parent_id);
          }
        }
      });
    });
  });
  
  
  // Bind accordian style
  $('.navigate-accordian-title').each(function () {
    if ($(this).hasClass('navigate-accordian-title-processed')) {
      return;
    }
    $(this).addClass('navigate-accordian-title-processed');
    $(this).click(function() {
      $(this).parents('.navigate-accordian').find('.navigate-accordian-content-outer').slideUp();
      $(this).parent().find('.navigate-accordian-content-outer').slideDown();
    });
  });
  
  
  // Bind actions to 'Load' for single users load / set options
  $('.navigate-default-load-user:not(.navigate-default-load-user-processed)').each(function () {
    $(this).addClass('navigate-default-load-user-processed');
    $(this).click(function () {
      Navigate.queueAdd();
      $.ajax({
        url: Navigate.ajaxUrl(),
        type: 'POST',
        data: 'action=defaults_load&username=' + $('#navigate-set-username').val(),
        error: function() {
        },
        success: function(msg) {
          $('.navigate-defaults').html(msg);
          Navigate.reloadWidgets();
          $('.navigate-defaults').animate({'opacity': 1.0});
          Navigate.queueSubtract();
          Drupal.attachBehaviors('.navigate-defaults');
        }
      });
    });
  });
  
  
  // Bind actions to 'Load' for single users load / set options
  $('.navigate-default-set-user:not(.navigate-default-set-user-processed)').each(function () {
    $(this).addClass('navigate-default-set-user-processed');
    $(this).click(function () {
      if (!confirm('Doing this will erase the current custom set for this user. Are you sure you want to continue?')) {
        return false;
      }
      Navigate.queueAdd();
      $.ajax({
        url: Navigate.ajaxUrl(),
        type: 'POST',
        data: 'action=defaults_set_all&username=' + $('#navigate-set-username').val(),
        error: function() {
        },
        success: function(msg) {
          $('.navigate-defaults').html(msg);
          Navigate.reloadWidgets();
          $('.navigate-defaults').animate({'opacity': 1.0});
          Navigate.queueSubtract();
          Drupal.attachBehaviors('.navigate-defaults');
        }
      });
    });
  });
  
  // Bind keyboard shortcut enable / disable
  $('.navigate-key-button:not(.navigate-key-button-processed)').each(function () {
    $(this).addClass('navigate-key-button-processed');
    
    $(this).click(function () {
      if ($(this).hasClass('key-disabled')) {
        $(document).bind('keydown', {combi:'ctrl+shift+n'});
        $(this).removeClass('key-disabled');
        Navigate.variableSet('key_disabled', '0');
      } else {
        $(document).unbind('keydown', {combi:'ctrl+shift+n'});
        $(this).addClass('key-disabled');
        Navigate.variableSet('key_disabled', '1');
      }
    });
  });
  
  // Bind import / export button
  $('.navigate-import-button:not(.navigate-import-button-procesed)').each(function () {
    $(this).click(function () {
      
    });
  });
  
  
}


/**
 * Set default item to bind events to
 *
 * If 'item' is not set, this will return .navigate, the class of the div containing the entire navigation
 */
function navigate_item_default(item) {
  item = typeof(item) != 'undefined' ? item : '.navigate ';
  return item + ' ';
}



/**
 * Hide / show navigate bar
 */
Navigate.toggle = function () {
  // Close
  if (navigate_on == 1) {
    Navigate.hide();
  } else {
    // Open
   Navigate.show();
  }
}

/**
 * Hide the navigate bar
 */
Navigate.hide = function () {
  if (navigate_on == 1) {
    navigate_on = 0;
    $(".navigate").stop();
    $("body").animate({ paddingLeft: "0px"}, "fast");
    $(".navigate").animate({ marginLeft: "-550px"}, "fast", function () {Navigate.variableSet('on', '0');});
    $("#navigate-switch").animate({ marginLeft: "-230px", marginTop: "-30px"}, "fast");
    $(".navigate-loading").animate({ marginLeft: "-550px"}, "fast");
    $(".navigate-admin-tools-outer").animate({ marginLeft: "-800px"}, "fast");
  }
}

/**
 * Show the navigate bar
 */
Navigate.show = function () {
  if (navigate_on != 1) {
    navigate_on = 1;
    Navigate.variableSet('on', '1');
    $("body").stop();
    $("body").animate({ paddingLeft: "210px"}, "fast");
    $(".navigate").animate({ marginLeft: "-195px"}, "fast");
    $("#navigate-switch").animate({ marginLeft: "-195px", marginTop: "0px"}, "fast");
    $(".navigate-loading").animate({ marginLeft: "-30px"}, "fast");
    $(".navigate-admin-tools-outer").animate({ marginLeft: "0px"}, "fast");
  }
}


/**
 * Binds events and various errata after page is loaded
 */
Navigate.bindNavigateOneTime = function () {

   // Make widgets sortable using their title as the handle
  $(".navigate-all-widgets-sortable").sortable({
      cursor: "move",
      distance: 10,
      handle: ".navigate-widget-title",
      revert:true,
      stop:function(e, ui) {
        Navigate.queueAdd();
        $.ajax({
          url: Navigate.ajaxUrl(),
          type: 'POST',
          data: 'action=widget_sort&' + $(".navigate-all-widgets").sortable('serialize'),
          error: function() {
          },
          success: function(msg) {
            Navigate.queueSubtract();
          }
        });
      }
    });
 
   // Bind click event to setting button to show close buttons and new widgets list
  $('.navigate-launch-settings').click(function() {
    $(".navigate-admin-tools-outer").slideToggle('fast', function() { $(this).animate({'opacity': 0.9});});
    $('.navigate-add-widgets').slideToggle('fast', function() {
      $('.navigate-widget-close').toggle();
    });
  });

  // Add select all on text inputs with select all specified
  $(".navigate-select-all").focus(function () {
    $(this).select();
  });
  
  // Set our navigate_on variable
  navigate_on = Drupal.settings.navigateOn;
  
  // Set initial styling.
  if (navigate_on == 0) {
    $("body").css("padding-left", "0px");
  } else {
    $("body").css("padding-left", "210px");
    $(".navigate").css("margin-left", "-195px");
    $("#navigate-switch").css("margin-left", "-195px").css("margin-top", "0px");
    $(".navigate-loading").css("margin-left", "-30px");
  }

  $("#navigate-switch").click(Navigate.toggle);


  // Bind click event to acklowledgement link
  $(".navigate-acknowledgement-switch").click(function () {
    $('.navigate-acknowledgement').slideToggle('fast');
  });

  // Bind doubleclick event to title
  $('.navigate-title').dblclick(function() {
    $('.navigate-shorten').slideToggle('fast');
  });
  
  // Bind help button
  $('.navigate-help-button').click(function () { window.location = $(this).find('.value').val();});
}


/**
 * Bind click actions to on/off buttons.
 */
Navigate.bindToggleButtons = function () {
  $('.navigate-button-outer').each(function() {
    if (!$(this).hasClass('toggle-button-processed')) {
      $(this).addClass('toggle-button-processed');
      var wid = Navigate.getWid(this);
      var widget = this;
      $(this).children().click(function() {
  
        // Grab all variables from hidden inputs
        var on = $(widget).find('.on').val();
        var off = $(widget).find('.off').val();
        var name = $(widget).find('.name').val();
        var required = $(widget).find('.required').val();
        var callback = $(widget).find('.callback').val();
        var val;
  
        // If it's a group
        if ($(widget).find('.group').length > 0) {
          var group = $(widget).find('.group').val();
          if ($(this).hasClass('navigate-button-on')) {
            if (required == 1) {
              return;
            }
            $(this).parents('.navigate-widget').find('.' + group).each(function(child) {
              $(this).removeClass('navigate-button-on');
            });
            $(this).removeClass('navigate-button-on');
            Navigate.variableSet(name, '', wid);
            return;
          }
          $(this).parents('.navigate-widget-outer').find('.' + group).each(function(child) {
            $(this).removeClass('navigate-button-on');
          });
          val = on;
          $(this).addClass('navigate-button-on');
  
        // If not a group
        } else {
          // If it's just a callback
          if (on != '') {
            if ($(this).hasClass('navigate-button-on')) {
              val = off;
              $(this).removeClass('navigate-button-on');
            } else {
              val = on;
              $(this).addClass('navigate-button-on');
            }
          }
        }
        if (callback != '' && callback != undefined) {
          window[callback](wid);
        }
        Navigate.variableSet(name, val, wid);
      });
    }
  });
}


/**
 * Bind callback click actions to buttons.
 */
Navigate.bindCallbackButtons = function () {
  $('.navigate-click-outer').each(function() {
    if ($(this).hasClass('navigate-click-outer-processed')) {
      return;
    }
    $(this).addClass('navigate-click-outer-processed');
    var wid = Navigate.getWid(this);
    var widget = this;
    $(this).children().click(function() {
      var callback = $(widget).children('.callback').val();
      if (callback != '' && callback != undefined) {
        window[callback](wid);
      }
    });
  });
}


/**
 * Bind text input events, including value saving and callback on pressing enter
 */
Navigate.bindTextInputs = function () {
  $('.navigate-text-input-outer').each(function() {
    if ($(this).hasClass('navigate-text-input-processed')) {
      return;
    }
    $(this).addClass('navigate-text-input-processed');
    var wid = Navigate.getWid(this);
    var widget = this;
    var callback = $(widget).children('.callback').val();
    var name = $(widget).children('.name').val();
    var text_input = this;

    $(this).find('input').bind('keyup', function(e) {
      var input_name = $(text_input).children('div').children('input').attr('name');
      var id = $(text_input).children('div').children('input').attr('id');
      var holder_id = name + wid + '_holder';
      // If pressing enter
      if (e.which == 13 || e.keyCode == 13) {
        if (callback != '' && callback != undefined) {
          window[callback](wid);
          if ($(this).hasClass('navigate-clear')) {
            var val = '';
            $(this).val('');
          }
        }
      } 
    });

  });
}


/**
 * Bind textarea save event, including value saving and callback on pressing enter
 */
Navigate.bindTextareas = function () {
  $('.navigate-textarea-outer').each(function() {
    if ($(this).hasClass('navigate-textarea-outer-processed')) {
      return;
    }
    $(this).addClass('navigate-textarea-outer-processed');
    var wid = Navigate.getWid(this);
    var widget = this;
    var callback = $(widget).children('.callback').val();
    var name = $(widget).children('.name').val();
    var text_input = this;
    $(this).find('.navigate-submit').click(function() {
      var val = $(widget).find('.navigate-textarea').val();
      Navigate.variableSet(name, val, wid);
      $(widget).find('.navigate-textarea').html(val);
      if (callback != '' && callback != undefined) {
        window[callback](wid);
      }
    });

  });
}


/**
 * Bind textarea save event, including value saving and callback on pressing enter
 */
Navigate.bindAdminTools = function () {
  if ($('.navigate-admin-tools-defaults').hasClass('navigate-admin-tools-defaults-processed')) {
    return;
  }
  $('.navigate-admin-tools-defaults').addClass('navigate-admin-tools-defaults-processed');
  
  // Set default
  $('.navigate-default-set').click(function () {
    $(this).addClass('navigate-default-set-processed');
    // Save the variable in a database
    Navigate.queueAdd();
    $('.navigate-defaults').animate({'opacity': 0.4});
    $.ajax({
       url: Navigate.ajaxUrl(),
       type: 'POST',
       data: 'action=save_defaults&rid=' + $(this).attr('rel'),
       error: function() {
       },
       success: function(msg) {
        $('.navigate-defaults').html(msg);
        $('.navigate-defaults').animate({'opacity': 1.0});
        Navigate.queueSubtract();
        Drupal.attachBehaviors('.navigate-defaults');
       }
    });
    return false;
  });
  
  // Unset default
  $('.navigate-default-unset').click(function () {
    // Save the variable in a database
    Navigate.queueAdd();
    $('.navigate-defaults').animate({'opacity': 0.4});
    $.ajax({
       url: Navigate.ajaxUrl(),
       type: 'POST',
       data: 'action=defaults_unset&rid=' + $(this).attr('rel'),
       error: function() {
       },
       success: function(msg) {
        $('.navigate-defaults').html(msg);
        $('.navigate-defaults').animate({'opacity': 1.0});
        Navigate.queueSubtract();
        Drupal.attachBehaviors('.navigate-defaults');
       }
    });
    return false;
  });
  
  // Load default
  $('.navigate-default-load').click(function () {
    // Save the variable in a database
    Navigate.queueAdd();
    $('.navigate-defaults').animate({'opacity': 0.4});
    $.ajax({
       url: Navigate.ajaxUrl(),
       type: 'POST',
       data: 'action=defaults_load&rid=' + $(this).attr('rel'),
       error: function() {
       },
       success: function(msg) {
        $('.navigate-defaults').html(msg);
        Navigate.reloadWidgets();
        $('.navigate-defaults').animate({'opacity': 1.0});
        Navigate.queueSubtract();
        Drupal.attachBehaviors('.navigate-defaults');
       }
    });
    return false;
  });
  
  // Switch back to saved set
  $('.navigate-default-switch-back').click(function () {
    // Save the variable in a database
    Navigate.queueAdd();
    $('.navigate-defaults').animate({'opacity': 0.4});
    $.ajax({
       url: Navigate.ajaxUrl(),
       type: 'POST',
       data: 'action=defaults_switch_back',
       error: function() {
       },
       success: function(msg) {
        $('.navigate-defaults').html(msg);
        Navigate.reloadWidgets();
        $('.navigate-defaults').animate({'opacity': 1.0});
        Navigate.queueSubtract();
        Drupal.attachBehaviors('.navigate-defaults');
       }
    });
    return false;
  });
  
  // Switch back to saved set
  $('.navigate-default-set-all').click(function () {
    if (!confirm('Doing this will erase the current custom set for all users in this role, and will set this set as the default for this role. Are you sure you want to continue?')) {
      return false;
    }
    // Save the variable in a database
    Navigate.queueAdd();
    $('.navigate-defaults').animate({'opacity': 0.4});
    $.ajax({
       url: Navigate.ajaxUrl(),
       type: 'POST',
       data: 'action=defaults_set_all&rid=' + $(this).attr('rel'),
       error: function() {
       },
       success: function(msg) {
        $('.navigate-defaults').html(msg);
        Navigate.reloadWidgets();
        $('.navigate-defaults').animate({'opacity': 1.0});
        Navigate.queueSubtract();
        Drupal.attachBehaviors('.navigate-defaults');
       }
    });
    return false;
  });
  
  // Bind user search check
  $('#navigate-set-username').bind('keyup',function () {
    if ($(this).val() != '') {
      $('.navigate-default-set-user-load').html('<a class="navigate-default-load-user" href="javascript:;">' + Drupal.t('Load') + '</a>');
      $('.navigate-default-set-user-set').html('<a class="navigate-default-set-user" href="javascript:;">' + Drupal.t('Set user') + '</a>');
    } else {
      $('.navigate-default-set-user-load').html('<span class="default-disabled">' + Drupal.t('Load') + '</span>');
      $('.navigate-default-set-user-set').html('<span class="default-disabled">' + Drupal.t('Set user') + '</span>');
    }
    Drupal.attachBehaviors('.navigate-defaults-table');
  });
  
  // Add instructions to empty username search
  $('#navigate-set-username').focus(function () {
    $(this).removeClass('navigate-username-empty');
    if ($(this).val() == Drupal.t('Search name / UID')) {
      $(this).val('');
    }
  });
  
  // Add instructions to empty username search
  $('#navigate-set-username').blur(function () {
    if ($(this).val() == '') {
      $(this).addClass('navigate-username-empty');
      $(this).val(Drupal.t('Search name / UID'));
    }
  });
}


/**
 * Reload navigate widget set
 */
Navigate.reloadWidgets = function () {
  $('.navigate-all-widgets').animate({'opacity': 0.4});
  $.ajax({
      url: Navigate.ajaxUrl(),
      type: 'POST',
      data: 'action=widgets_reload',
      error: function() {
      },
      success: function(msg) {
       $('.navigate-all-widgets').html(msg);
       $('.navigate-all-widgets').animate({'opacity': 1.0});
       Navigate.queueSubtract();
       $('.navigate-widget-close').show();
       Drupal.attachBehaviors('.navigate-all-widgets');
      }
   });
}

/**
 * Set a variable
 */
Navigate.variableSet = function (name, val, wid) {
  // Save the variable in a database
  Navigate.queueAdd();
   $.ajax({
      url: Navigate.ajaxUrl(),
      type: 'POST',
      data: 'action=variable_save&name=' + name + '&wid=' + wid + '&value=' + encodeURIComponent(val),
      error: function() {
      },
      success: function(msg) {
        Navigate.queueSubtract();
      }
  });
}

/**
 * Add count to loader queue
 */
Navigate.queueAdd = function () {
  if (Navigate.loaderCount < 1) {
    $(".navigate-loading").show();
  }
  Navigate.loaderCount++;
}

/**
 * Remove count from loader
 */
Navigate.queueSubtract = function () {
  Navigate.loaderCount--;
  if (Navigate.loaderCount < 1) {
    $(".navigate-loading").slideUp();
  }
}


/**
 * Grab the wid of a widget from inside the widget
 */
Navigate.getWid = function (item) {
  return $(item).parents('.navigate-widget-outer').children('.wid').val();
}


/**
 * Get the location of the current URL
 */
Navigate.ajaxUrl = function () {
  return Drupal.settings.basePath + 'navigate/process'
}


/**
 * Send request to navigate processing function
 *
 * This is used to run widget callbacks through, so we can sanitize query string data.
 */
Navigate.processAjax = function (wid, query_array, callback) {
  var query_string = '';
  for(var i in query_array) {
    if (i != '') {
      query_string += i + '=';
    }
    query_string += query_array[i] + '&';
  }
  query_string += '&wid=' + wid + '&return=' + $('#navigate_q').val();
  Navigate.queueAdd();
  $.ajax({
      url: Navigate.ajaxUrl(),
      type: 'POST',
      data: query_string,
      error: function() {
      },
      success: function(msg) {
        Navigate.queueSubtract();
        if (callback != undefined) {
          window[callback](msg, wid);
        }
      }
  });
}


/**
 * Binds click event to list of widgets
 */
Navigate.bindAddWidgetList = function () {
  if ($('.navigate-widget-list').hasClass('navigate-widget-list-processed')) {
    return;
  }
  $('.navigate-widget-list').addClass('navigate-widget-list-processed');
  $('.navigate-widget-list-item').click(function() {
    $list_item = $(this);
    
    $('.navigate-widget-list-item').addClass('navigate-widget-list-processed');
    
    // If it's a one-at-a-time widget, return false
    if ($(this).find('.single').val() == 1) {
      if ($('.navigate-widget-outer > .type[value="' + $(this).attr('id') + '"]').length > 0) {
        alert(t('Sorry, only one instance of this widget is allowed at one time.'));
        return false;
      }
    }
    Navigate.queueAdd();
    $.ajax({
      url: Navigate.ajaxUrl(),
      type: 'POST',
      dataType: 'html',
      data: 'action=add_widget&module=' + $(this).attr('title') + '&type=' + $(this).attr('id'),
      error: function() {
      },
      success: function(msg) {
        Navigate.queueSubtract();
        $('.navigate-all-widgets').append(msg);
        var widget = '#' + $(msg).parent().children('.navigate-widget-outer').attr('id');
        funct = $list_item.find('.callback').val();
        if (funct != '' && funct != undefined) {
          window[funct](widget);
        }
        Drupal.attachBehaviors();
      }
    });
    return true;
  });
}

/**
 * Create export content and populate textarea
 */
function navigate_set_export() {
  $('.navigate-import-export').animate({'opacity': 0.4});
  $.ajax({
    url: Navigate.ajaxUrl(),
    type: 'POST',
    dataType: 'html',
    data: 'action=export_widget_set',
    error: function() {
    },
    success: function(msg) {
      Navigate.queueSubtract();
      $('#navigate-favorites-export_').val(msg);
      $('.navigate-import-export').animate({'opacity': 1.0});
      Drupal.attachBehaviors();
    }
  });
}

/**
 * Create export content and populate textarea
 */
function navigate_set_import() {
  $('.navigate-import-export').animate({'opacity': 0.4});
  $.ajax({
    url: Navigate.ajaxUrl(),
    type: 'POST',
    dataType: 'html',
    data: 'action=import_widget_set&import=' + $('#navigate-favorites-import_').val(),
    error: function() {
    },
    success: function(msg) {
      Navigate.queueSubtract();
      $('#navigate-favorites-import_').val('');
      $('.navigate-import-export').animate({'opacity': 1.0});
      Navigate.reloadWidgets();
      Drupal.attachBehaviors();
    }
  });
}

